@@ -18,6 +18,7 @@
 #include <qclipboard.h>
 #include <qmime.h>
 #include <qspinbox.h>
+#include <qevent.h>
 #include <kapplication.h>
 #include <kdebug.h>
 #include <kruler.h>
@@ -54,12 +55,15 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
     m_doc(doc),
     m_pasteData(0),
     m_scaleFormat(Stitches),
-    m_updatingDisplay(false)
+    m_updatingDisplay(false),
+    m_points(0)
 {
   KActionCollection* actionCollection = ((KXStitchApp*)topLevelWidget())->actionCollection();
   KIconLoader *loader = KGlobal::iconLoader();
   KRadioAction* radioAction;
 
+  m_editUndo = KStdAction::undo(this, SLOT(slotEditUndo()), actionCollection);
+  m_editRedo = KStdAction::redo(this, SLOT(slotEditRedo()), actionCollection);
   m_editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection);
   m_editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection);
   m_editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection);
@@ -72,6 +76,8 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   new KAction(i18n("Insert text..."), QIconSet(loader->loadIcon("text",KIcon::Toolbar)), 0,this,SLOT(slotTextTool()),actionCollection,"Tool_text");
   new KAction(i18n("Insert library pattern..."),0,this,SLOT(slotLibraryPattern()),actionCollection,"librarypattern");
   m_createLibraryPattern = new KAction(i18n("Create library pattern..."),0,this,SLOT(slotLibraryCreateLibraryPattern()),actionCollection,"createlibrarypattern");
+  new KAction(i18n("Export library patterns..."),0,this,SLOT(slotLibraryExport()),actionCollection,"exportlibrary");
+  new KAction(i18n("Import library patterns..."),0,this,SLOT(slotLibraryImport()),actionCollection,"importlibrary");
   KStdAction::fitToPage(this, SLOT(slotFitPage()), actionCollection);
   KStdAction::fitToWidth(this, SLOT(slotFitWidth()), actionCollection);
   KStdAction::fitToHeight(this, SLOT(slotFitHeight()), actionCollection);
@@ -160,6 +166,8 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
 
   new KToggleAction(i18n("Stitch Mask"), 0, this, SLOT(slotStitchMask()), actionCollection, "StitchMask");
   new KToggleAction(i18n("Color Mask"), 0, this, SLOT(slotColorMask()), actionCollection, "ColorMask");
+  new KToggleAction(i18n("Exclude Backstitches"), 0, this, SLOT(slotExcludeBackstitches()), actionCollection, "ExcludeBackstitches");
+  new KToggleAction(i18n("Exclude Knots"), 0, this, SLOT(slotExcludeKnots()), actionCollection, "ExcludeKnots");
 
   configure(true);
 
@@ -167,9 +175,13 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   setVScrollBarMode(AlwaysOn);
   setBackgroundMode(PaletteBase);
   viewport()->setBackgroundMode(NoBackground);
+  viewport()->setFocusPolicy(QWidget::StrongFocus);
+  viewport()->setMouseTracking(true);
   m_currentStitchType = Stitch::Full;
   m_stitchMask=false;
   m_colorMask=false;
+  m_excludeBackstitches=false;
+  m_excludeKnots=false;
 
   slotClipboardDataChanged();
   connect(kapp->clipboard(),SIGNAL(dataChanged()),this,SLOT(slotClipboardDataChanged()));
@@ -254,15 +266,25 @@ void EditView::slotFitBackgroundToSelection()
   updateContents();
 }
 
+void EditView::slotEditUndo()
+{
+  m_doc->undo();
+}
+
+void EditView::slotEditRedo()
+{
+  m_doc->redo();
+}
+
 void EditView::slotEditCut()
 {
-  kapp->clipboard()->setData(new KXStitchMimeSource(m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,true)));
+  kapp->clipboard()->setData(new KXStitchMimeSource(m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,m_excludeBackstitches,m_excludeKnots,true)));
   clearSelection();
 }
 
 void EditView::slotEditCopy()
 {
-  kapp->clipboard()->setData(new KXStitchMimeSource(m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,false)));
+  kapp->clipboard()->setData(new KXStitchMimeSource(m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,m_excludeBackstitches,m_excludeKnots,false)));
   clearSelection();
 }
 
@@ -275,7 +297,7 @@ void EditView::slotEditPaste()
 
 void EditView::editRotate(int angle)
 {
-  QByteArray selection = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,!m_editMirrorRotateCopies->isChecked());
+  QByteArray selection = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,m_excludeBackstitches,m_excludeKnots,!m_editMirrorRotateCopies->isChecked());
   clearSelection();
 
   QDataStream stream(selection, IO_ReadOnly);
@@ -512,7 +534,7 @@ void EditView::slotEditRotate270()
 
 void EditView::editMirror(bool vertical)
 {
-  QByteArray selection = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,!m_editMirrorRotateCopies->isChecked());
+  QByteArray selection = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,m_excludeBackstitches,m_excludeKnots,!m_editMirrorRotateCopies->isChecked());
   clearSelection();
 
   QDataStream stream(selection, IO_ReadOnly);
@@ -711,15 +733,37 @@ void EditView::slotTextTool()
 
 void EditView::slotLibraryCreateLibraryPattern()
 {
-  QByteArray d = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,false);
-  PatternLibraryDialog* dialog = new PatternLibraryDialog(this,new KXStitchMimeSource(d));
+  QByteArray d = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,m_excludeBackstitches,m_excludeKnots,false);
+  PatternLibraryDialog* dialog = new PatternLibraryDialog(this, PatternLibraryDialog::Create, new KXStitchMimeSource(d));
   dialog->exec();
   clearSelection();
 }
 
 void EditView::slotLibraryPattern()
 {
-  PatternLibraryDialog* dialog = new PatternLibraryDialog(this);
+  PatternLibraryDialog* dialog = new PatternLibraryDialog(this, PatternLibraryDialog::Insert);
+  if (dialog->exec() == QDialog::Accepted)
+  {
+    m_pasteData = dialog->pattern();
+    m_pasteDataIsLibrary = true;
+    setMode(EditView::Paste);
+  }
+  delete dialog;
+}
+
+void EditView::slotLibraryExport()
+{
+  kdDebug() << "EditView::slotLibraryExport()" << endl;
+  PatternLibraryDialog* dialog = new PatternLibraryDialog(this,PatternLibraryDialog::Export);
+  dialog->exec();
+  delete dialog;
+}
+
+void EditView::slotLibraryImport()
+{
+  kdDebug() << "EditView::slotLibraryImport()" << endl;
+  // initially create the dialog for import, after import the mode is changed to allow selecting and insert
+  PatternLibraryDialog* dialog = new PatternLibraryDialog(this,PatternLibraryDialog::Import);
   if (dialog->exec() == QDialog::Accepted)
   {
     m_pasteData = dialog->pattern();
@@ -907,8 +951,8 @@ int EditView::print(KPrinter* printer, QPainter* painter, int width, int height,
       wrapCellsTall = patternHeight % pageCellsTall;
       // do some checks here to see if there are odd lines of the pattern below a define minimum going
       // to be printed on seperate sheets, then modify printCellWidth to bring them onto the previous page
-      if (wrapCellsWide && (wrapCellsWide < cellsWrapMin) && (printPagesWide > 1)) pageLayoutOK = false;
-      if (wrapCellsTall && (wrapCellsTall < cellsWrapMin) && (printPagesTall > 1)) pageLayoutOK = false;
+      if (wrapCellsWide && (wrapCellsWide < cellsWrapMin) && (printPagesWide > 0)) pageLayoutOK = false;
+      if (wrapCellsTall && (wrapCellsTall < cellsWrapMin) && (printPagesTall > 0)) pageLayoutOK = false;
       if (!pageLayoutOK) printCellSize--;
     } while (!pageLayoutOK);
     if (wrapCellsWide) printPagesWide++;
@@ -958,6 +1002,32 @@ void EditView::slotSetCenter(QPoint pos)
   emit changedVisibleCells(visibleCells());
 }
 
+void EditView::keyPressEvent(QKeyEvent* event)
+{
+  if (m_mode == FillPolyline && m_points)
+  {
+    switch (event->key())
+    {
+      case Qt::Key_Backspace: // delete the last selected vertex
+        m_points--;
+        event->accept();
+        break;
+
+      case Qt::Key_Escape: // cancel the operation
+        m_points = 0;
+        event->accept();
+        break;
+
+      default:
+        event->ignore();
+        break;
+    }
+    updateContents();
+  }
+  else
+    event->ignore();
+}
+
 void EditView::paintEvent(QPaintEvent *event)
 {
   PatternCanvas* pCanvas = m_doc->canvas();
@@ -1178,6 +1248,7 @@ void EditView::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cl
     sym.setFamily(KXSConfig().Editor_SymbolFont);
     sym.setPointSize(72*m_cellSize/dpix);
     doubleBuffer->setFont(sym);
+    initialiseRenderCellsFonts(doubleBuffer,m_cellSize);
     for (int yCell = s.y() ; yCell < e.y() ; yCell++)
     {
       for (int xCell = s.x() ; xCell < e.x() ; xCell++)
@@ -1186,10 +1257,14 @@ void EditView::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cl
         QRect cellRect(cellToRect(cell));
         if (clip.intersects(cellRect))
         {
-          renderCell(doubleBuffer, cellRect, pCanvas->stitchAt(cell),m_vmode);
+          doubleBuffer->resetXForm();
+          doubleBuffer->translate(cellRect.left()-clipx,cellRect.top()-clipy);
+          renderCell(doubleBuffer,pCanvas->stitchAt(cell),m_vmode);
         }
       }
     }
+    doubleBuffer->resetXForm();
+    doubleBuffer->translate(-clipx,-clipy);
     sym.setPointSize(sym.pointSize()/2);
     doubleBuffer->setFont(sym);
     QPtrListIterator<Knot> *itK = pCanvas->knots();
@@ -1229,7 +1304,6 @@ void EditView::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cl
           break;
       }
     }
-    doubleBuffer->setFont(fnt);
   }
 
   if (m_backstitches)
@@ -1244,12 +1318,23 @@ void EditView::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cl
     }
     delete it;
   }
+  if (m_points)
+  {
+    doubleBuffer->setBrush(Qt::NoBrush);
+    doubleBuffer->setPen(Qt::red);    // perhaps make this user defined
+    doubleBuffer->drawEllipse(cellToRect(m_pointArray[0]));
+    doubleBuffer->setPen(Qt::green);  // perhaps make this user defined
+    for (int i = 1 ; i < m_points ; i++)
+    {
+      doubleBuffer->drawEllipse(cellToRect(m_pointArray[i]));
+      doubleBuffer->drawLine((cellToRect(m_pointArray[i-1]).center()),(cellToRect(m_pointArray[i]).center()));
+    }
+  }
   doubleBuffer->end();
   p->drawPixmap(clipx,clipy,*buffer);
   delete doubleBuffer;
   delete buffer;
 
-//  showSelectionRect();
   m_updatingDisplay = false;
 
   // update the preview location marker
@@ -1268,6 +1353,7 @@ void EditView::printContents(KPrinter *printer, QPainter *painter, int x, int y,
   sym.setFamily(KXSConfig().Editor_SymbolFont);
   sym.setPointSize(72*printCellSize/dpix);
   painter->setFont(sym);
+  initialiseRenderCellsFonts(painter,printCellSize);
 
   if (printer->option("kde-kxstitch-gridlines") == "true")
   {
@@ -1312,13 +1398,17 @@ void EditView::printContents(KPrinter *printer, QPainter *painter, int x, int y,
   if (stitchesAs == i18n("Colored Blocks & Symbols")) sViewMode = ColorBlocksSymbols;
   if (stitchesAs != i18n("None"))
   {
+    painter->save();
     for (int dy = 0 ; dy < pageCellsTall ; dy++)
     {
       for (int dx = 0 ; dx < pageCellsWide ; dx++)
       {
-        renderCell(painter,QRect(printable.left()+(dx*printCellSize),printable.top()+(dy*printCellSize),printCellSize,printCellSize),pCanvas->stitchAt(QPoint(x+dx,y+dy)),sViewMode);// render the cell
+        painter->translate(printable.left()+(dx*printCellSize),printable.top()+(dy*printCellSize));
+        renderCell(painter,pCanvas->stitchAt(QPoint(x+dx,y+dy)),sViewMode);// render the cell
+        painter->resetXForm();
       }
     }
+    painter->restore();
   }
 
   FlossPalette *pPalette = m_doc->palette();
@@ -1410,6 +1500,8 @@ void EditView::contentsMousePressEvent(QMouseEvent *e)
   {
     return;
   }
+  if (m_mode != FillPolyline || m_points == 0)
+    m_doc->createSavePoint();
   QRect rect;
   QPoint p = e->pos();
   if (m_selectionTimer.isActive())
@@ -1561,6 +1653,12 @@ void EditView::contentsMouseMoveEvent(QMouseEvent *e)
   QPainter painter;
   QPoint p = e->pos();
   m_tracking = contentsToCell(p);
+  if (e->state() == Qt::NoButton)
+  {
+    // this is a mouse move event only, no command is currently in operation
+    emit locationOrSize(m_tracking);
+    return;
+  }
   switch (m_mode)
   {
     case Paint:
@@ -1593,6 +1691,7 @@ void EditView::contentsMouseMoveEvent(QMouseEvent *e)
       {
         drawBoundingLine(contentsToViewport(cellToRect(m_start).center()),contentsToViewport(cellToRect(m_end).center()));
         drawBoundingLine(contentsToViewport(cellToRect(m_start).center()),contentsToViewport(cellToRect(m_tracking).center()));
+        ensureVisible(p.x(),p.y());
         m_end = m_tracking;
         rect = QRect(m_start,m_end).normalize();
         emit locationOrSize(QPoint(rect.width(),rect.height()));
@@ -1643,6 +1742,7 @@ void EditView::contentsMouseMoveEvent(QMouseEvent *e)
       if (m_tracking != m_end)
       {
         drawBoundingRect(m_start, m_end);
+        ensureVisible(p.x(),p.y());
         drawBoundingRect(m_start, m_tracking);
         m_end = m_tracking;
         rect = QRect(m_start,m_end).normalize();
@@ -1652,6 +1752,7 @@ void EditView::contentsMouseMoveEvent(QMouseEvent *e)
     case SelectRectangle:
       drawSelectionRectangle();
       m_end = m_tracking;
+      ensureVisible(p.x(),p.y());
       drawSelectionRectangle();
       rect = QRect(m_start,m_end).normalize();
       emit locationOrSize(QPoint(rect.width(),rect.height()));
@@ -1661,6 +1762,7 @@ void EditView::contentsMouseMoveEvent(QMouseEvent *e)
       if (m_tracking != m_end)
       {
         drawBoundingEllipse(m_start, m_end);
+        ensureVisible(p.x(),p.y());
         drawBoundingEllipse(m_start, m_tracking);
         m_end = m_tracking;
         rect = QRect(m_start,m_end).normalize();
@@ -1863,12 +1965,6 @@ void EditView::contentsMouseReleaseEvent(QMouseEvent *e)
             m_pointArray.resize(0);
             m_points = 0;
           }
-          else{
-            if (m_points > 1)
-            {
-              drawBoundingLine(contentsToViewport(cellToRect(m_pointArray[m_points-2]).center()),contentsToViewport(cellToRect(m_pointArray[m_points-1]).center()));
-            }
-          }
           updateContents();
           break;
         case SelectRectangle:
@@ -2257,6 +2353,18 @@ void EditView::slotColorMask()
   emit colorMask(m_colorMask);
 }
 
+void EditView::slotExcludeBackstitches()
+{
+  m_excludeBackstitches = !m_excludeBackstitches;
+  emit excludeBackstitches(m_excludeBackstitches);
+}
+
+void EditView::slotExcludeKnots()
+{
+  m_excludeKnots = !m_excludeKnots;
+  emit excludeKnots(m_excludeKnots);
+}
+
 void EditView::slotFitWidth()
 {
   m_cellSize = visibleWidth()/m_doc->canvas()->patternWidth();
@@ -2359,13 +2467,11 @@ void EditView::showSelectionRect()
 
 void EditView::slotScrolling(int, int)
 {
-//  hideSelectionRect();
   if (m_scales)
   {
     update(frameWidth(),frameWidth(),width(),m_scaleSize);
     update(frameWidth(),frameWidth(),m_scaleSize,height());
   }
-//  showSelectionRect();
 }
 
 void EditView::setMode(ToolMode m)
@@ -2390,14 +2496,12 @@ void EditView::setViewMode(ViewMode m)
 {
   m_vmode = m;
   updateContents();
-//viewport()->repaint(false);
 }
 
 void EditView::setBSViewMode(ViewMode m)
 {
   m_bsvmode = m;
   updateContents();
-//viewport()->repaint(false);
 }
 
 void EditView::drawBoundingRect(QPoint s, QPoint e)
@@ -2426,63 +2530,139 @@ void EditView::drawBoundingLine(QPoint s, QPoint e)
   painter.drawLine(s, e);
 }
 
-void EditView::renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches,ViewMode viewMode)
+void EditView::initialiseRenderCellsFonts(QPainter *p, int size)
+{
+  int half          = size/2;
+  int third         = size/3;
+
+  m_renderCell      = QRect(0,0,size,size);
+  m_renderTLCell    = QRect(0,0,half,half);
+  m_renderTL3Cell   = QRect(0,0,size-third,size-third);
+  m_renderTRCell    = QRect(half,0,half,half);
+  m_renderTR3Cell   = QRect(third,0,size-third,size-third);
+  m_renderBLCell    = QRect(0,half,half,half);
+  m_renderBL3Cell   = QRect(third,0,size-third,size-third);
+  m_renderBRCell    = QRect(half,half,half,half);
+  m_renderBR3Cell   = QRect(third,third,size-third,size-third);
+
+  m_renderTLQ[0]    = 0;
+  m_renderTLQ[1]    = 0;
+  m_renderTLQ[2]    = half;
+  m_renderTLQ[3]    = 0;
+  m_renderTLQ[4]    = 0;
+  m_renderTLQ[5]    = half;
+
+  m_renderTRQ[0]    = half;
+  m_renderTRQ[1]    = 0;
+  m_renderTRQ[2]    = size;
+  m_renderTRQ[3]    = half;
+  m_renderTRQ[4]    = size;
+  m_renderTRQ[5]    = half;
+
+  m_renderBLQ[0]    = 0;
+  m_renderBLQ[1]    = half;
+  m_renderBLQ[2]    = half;
+  m_renderBLQ[3]    = size;
+  m_renderBLQ[4]    = 0;
+  m_renderBLQ[5]    = size;
+
+  m_renderBRQ[0]    = size;
+  m_renderBRQ[1]    = half;
+  m_renderBRQ[2]    = size;
+  m_renderBRQ[3]    = size;
+  m_renderBRQ[4]    = half;
+  m_renderBRQ[5]    = size;
+
+  m_renderBLTRH[0]  = 0;
+  m_renderBLTRH[1]  = size;
+  m_renderBLTRH[2]  = 0;
+  m_renderBLTRH[3]  = half;
+  m_renderBLTRH[4]  = half;
+  m_renderBLTRH[5]  = 0;
+  m_renderBLTRH[6]  = size;
+  m_renderBLTRH[7]  = half;
+  m_renderBLTRH[8]  = size;
+  m_renderBLTRH[9]  = half;
+  m_renderBLTRH[10] = half;
+  m_renderBLTRH[11] = size;
+
+  m_renderTLBRH[0]  = 0;
+  m_renderTLBRH[1]  = 0;
+  m_renderTLBRH[2]  = half;
+  m_renderTLBRH[3]  = 0;
+  m_renderTLBRH[4]  = size;
+  m_renderTLBRH[5]  = half;
+  m_renderTLBRH[6]  = size;
+  m_renderTLBRH[7]  = size;
+  m_renderTLBRH[8]  = half;
+  m_renderTLBRH[9]  = size;
+  m_renderTLBRH[10] = 0;
+  m_renderTLBRH[11] = half;
+
+  m_renderTL3Q[0]   = 0;
+  m_renderTL3Q[1]   = 0;
+  m_renderTL3Q[2]   = size;
+  m_renderTL3Q[3]   = 0;
+  m_renderTL3Q[4]   = size;
+  m_renderTL3Q[5]   = half;
+  m_renderTL3Q[6]   = half;
+  m_renderTL3Q[7]   = size;
+  m_renderTL3Q[8]   = 0;
+  m_renderTL3Q[9]   = size;
+
+  m_renderTR3Q[0]   = 0;
+  m_renderTR3Q[1]   = 0;
+  m_renderTR3Q[2]   = size;
+  m_renderTR3Q[3]   = 0;
+  m_renderTR3Q[4]   = size;
+  m_renderTR3Q[5]   = size;
+  m_renderTR3Q[6]   = half;
+  m_renderTR3Q[7]   = size;
+  m_renderTR3Q[8]   = 0;
+  m_renderTR3Q[9]   = half;
+
+  m_renderBL3Q[0]   = 0;
+  m_renderBL3Q[1]   = 0;
+  m_renderBL3Q[2]   = half;
+  m_renderBL3Q[3]   = 0;
+  m_renderBL3Q[4]   = size;
+  m_renderBL3Q[5]   = half;
+  m_renderBL3Q[6]   = size;
+  m_renderBL3Q[7]   = size;
+  m_renderBL3Q[8]   = 0;
+  m_renderBL3Q[9]   = size;
+
+  m_renderBR3Q[0]   = half;
+  m_renderBR3Q[1]   = 0;
+  m_renderBR3Q[2]   = size;
+  m_renderBR3Q[3]   = 0;
+  m_renderBR3Q[4]   = size;
+  m_renderBR3Q[5]   = size;
+  m_renderBR3Q[6]   = 0;
+  m_renderBR3Q[7]   = size;
+  m_renderBR3Q[8]   = 0;
+  m_renderBR3Q[9]   = half;
+
+  m_renderFont      = p->font();
+  m_renderQtrFont   = m_renderFont;
+  m_render3QtrFont  = m_renderFont;
+
+  m_renderQtrFont.setPointSize(m_renderFont.pointSize()/2);
+  m_render3QtrFont.setPointSize(m_renderFont.pointSize()*3/4);
+}
+
+void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMode)
 {
   FlossPalette *pPalette = m_doc->palette();
 
-  int x = rect.left(),
-      y = rect.top(),
-      x1 = rect.right(),
-      y1 = rect.bottom();
-  QPoint c = rect.center();
+  int x = m_renderCell.left(),
+      y = m_renderCell.top(),
+      x1 = m_renderCell.right(),
+      y1 = m_renderCell.bottom();
+  QPoint c = m_renderCell.center();
   int cx = c.x(),
       cy = c.y();
 
-  QFont originalFont = p->font(); // this one set from either drawContents or printContents
-  QFont threeQuarterFont = originalFont;
-  threeQuarterFont.setPointSize(originalFont.pointSize()*3/4);
-  QFont quarterFont = originalFont;
-  quarterFont.setPointSize(originalFont.pointSize()/2);
-  QRect tlq(rect);
-  tlq.setRight(rect.center().x());
-  tlq.setBottom(rect.center().y());
-  QRect trq(rect);
-  trq.setLeft(rect.center().x());
-  trq.setBottom(rect.center().y());
-  QRect blq(rect);
-  blq.setTop(rect.center().y());
-  blq.setRight(rect.center().x());
-  QRect brq(rect);
-  brq.setLeft(rect.center().x());
-  brq.setTop(rect.center().y());
-  int third = rect.width()/3;
-  QRect tl3q(rect);
-  tl3q.rRight()-=third;
-  tl3q.rBottom()-=third;
-  QRect tr3q(rect);
-  tr3q.rLeft()+=third;
-  tr3q.rBottom()-=third;
-  QRect bl3q(rect);
-  bl3q.rTop()+=third;
-  bl3q.rRight()-=third;
-  QRect br3q(rect);
-  br3q.rLeft()+=third;
-  br3q.rTop()+=third;
-//  QRect displayRect;
-
-  QCOORD tlqC[6] = { x,y, cx,y, x,cy };
-  QCOORD trqC[6] = { cx,y, x1,y, x1,cy };
-  QCOORD blqC[6] = { x,cy, cx,y1, x,y1 };
-  QCOORD brqC[6] = { x1,cy, x1,y1, cx,y1 };
-
-  QCOORD bltrh[12] = { x,y1, x,cy, cx,y, x1,y, x1,cy, cx,y1 };
-  QCOORD tlbrh[12] = { x,y, cx,y, x1,cy, x1,y1, cx,y1, x,cy };
-
-  QCOORD tl3qC[10] = { x,y, x1,y, x1,cy, cx,y1, x,y1 };
-  QCOORD tr3qC[10] = { x,y, x1,y, x1,y1, cx,y1, x,cy };
-  QCOORD bl3qC[10] = { x,y, cx,y, x1,cy, x1,y1, x,y1 };
-  QCOORD br3qC[10] = { cx,y, x1,y, x1,y1, x,y1, x,cy };
-
   if (viewMode==BWSymbols)
     p->setPen(black);
   if (pStitches)
@@ -2500,176 +2680,169 @@ void EditView::renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches,View
       switch (cst)
       {
         case Stitch::TLQtr:
-          p->setFont(quarterFont);
+          p->setFont(m_renderQtrFont);
           switch(viewMode)
           {
             case Regular :
-              p->drawLine(x,y,cx,cy);
+              p->drawLine(0,0,cx,cy);
               break;
             case BWSymbols:
-              p->drawText(tlq, AlignCenter, s, 1);
+              p->drawText(m_renderTLCell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(tlq, AlignCenter, s, 1);
+              p->drawText(m_renderTLCell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-//              p->fillRect(tlq,c);
-              p->drawPolygon(QPointArray(3,tlqC));
+              p->drawPolygon(QPointArray(3,m_renderTLQ));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(tlq, AlignCenter, s, 1);
+                p->drawText(m_renderTLCell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::TRQtr:
-          p->setFont(quarterFont);
+          p->setFont(m_renderQtrFont);
           switch(viewMode)
           {
             case Regular :
               p->drawLine(x1,y,cx,cy);
               break;
             case BWSymbols:
-              p->drawText(trq, AlignCenter, s, 1);
+              p->drawText(m_renderTRCell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(trq, AlignCenter, s, 1);
+              p->drawText(m_renderTRCell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(3,trqC));
+              p->drawPolygon(QPointArray(3,m_renderTRQ));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(trq, AlignCenter, s, 1);
+                p->drawText(m_renderTRCell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::BLQtr:
-          p->setFont(quarterFont);
+          p->setFont(m_renderQtrFont);
           switch(viewMode)
           {
             case Regular :
               p->drawLine(x,y1,cx,cy);
               break;
             case BWSymbols:
-              p->drawText(blq, AlignCenter, s, 1);
+              p->drawText(m_renderBLCell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(blq, AlignCenter, s, 1);
+              p->drawText(m_renderBLCell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(3,blqC));
+              p->drawPolygon(QPointArray(3,m_renderBLQ));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(blq, AlignCenter, s, 1);
+                p->drawText(m_renderBLCell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::BRQtr:
-          p->setFont(quarterFont);
+          p->setFont(m_renderQtrFont);
           switch(viewMode)
           {
             case Regular :
               p->drawLine(x1,y1,cx,cy);
               break;
             case BWSymbols:
-              p->drawText(brq, AlignCenter, s, 1);
+              p->drawText(m_renderBRCell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(brq, AlignCenter, s, 1);
+              p->drawText(m_renderBRCell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(3,brqC));
+              p->drawPolygon(QPointArray(3,m_renderBRQ));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(brq, AlignCenter, s, 1);
+                p->drawText(m_renderBRCell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::BTHalf:
-          p->setFont(threeQuarterFont);
+          p->setFont(m_render3QtrFont);
           switch(viewMode)
           {
             case Regular :
               p->drawLine(x1,y,x,y1);
               break;
             case BWSymbols:
-              p->drawText(rect, AlignCenter, s, 1);
+              p->drawText(m_renderCell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(rect, AlignCenter, s, 1);
+              p->drawText(m_renderCell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(6,bltrh));
+              p->drawPolygon(QPointArray(6,m_renderBLTRH));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(rect, AlignCenter, s, 1);
+                p->drawText(m_renderCell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::TBHalf:
-          p->setFont(threeQuarterFont);
+          p->setFont(m_render3QtrFont);
           switch(viewMode)
           {
             case Regular :
               p->drawLine(x,y,x1,y1);
               break;
             case BWSymbols:
-              p->drawText(rect, AlignCenter, s, 1);
+              p->drawText(m_renderCell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(rect, AlignCenter, s, 1);
+              p->drawText(m_renderCell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(6,tlbrh));
+              p->drawPolygon(QPointArray(6,m_renderTLBRH));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(rect, AlignCenter, s, 1);
+                p->drawText(m_renderCell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::TL3Qtr:
-          p->setFont(threeQuarterFont);
+          p->setFont(m_render3QtrFont);
           switch(viewMode)
           {
             case Regular :
@@ -2677,28 +2850,27 @@ void EditView::renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches,View
               p->drawLine(x,y,cx,cy);
               break;
             case BWSymbols:
-              p->drawText(tl3q, AlignCenter, s, 1);
+              p->drawText(m_renderTL3Cell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(tl3q, AlignCenter, s, 1);
+              p->drawText(m_renderTL3Cell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(5,tl3qC));
+              p->drawPolygon(QPointArray(5,m_renderTL3Q));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(tl3q, AlignCenter, s, 1);
+                p->drawText(m_renderTL3Cell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::TR3Qtr:
-          p->setFont(threeQuarterFont);
+          p->setFont(m_render3QtrFont);
           switch(viewMode)
           {
             case Regular :
@@ -2706,28 +2878,27 @@ void EditView::renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches,View
               p->drawLine(x1,y,cx,cy);
               break;
             case BWSymbols:
-              p->drawText(tr3q, AlignCenter, s, 1);
+              p->drawText(m_renderTR3Cell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(tr3q, AlignCenter, s, 1);
+              p->drawText(m_renderTR3Cell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(5,tr3qC));
+              p->drawPolygon(QPointArray(5,m_renderTR3Q));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(tr3q, AlignCenter, s, 1);
+                p->drawText(m_renderTR3Cell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::BL3Qtr:
-          p->setFont(threeQuarterFont);
+          p->setFont(m_render3QtrFont);
           switch(viewMode)
           {
             case Regular :
@@ -2735,28 +2906,27 @@ void EditView::renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches,View
               p->drawLine(x,y1,cx,cy);
               break;
             case BWSymbols:
-              p->drawText(bl3q, AlignCenter, s, 1);
+              p->drawText(m_renderBL3Cell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(bl3q, AlignCenter, s, 1);
+              p->drawText(m_renderBL3Cell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(5,bl3qC));
+              p->drawPolygon(QPointArray(5,m_renderBL3Q));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(bl3q, AlignCenter, s, 1);
+                p->drawText(m_renderBL3Cell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::BR3Qtr:
-          p->setFont(threeQuarterFont);
+          p->setFont(m_render3QtrFont);
           switch(viewMode)
           {
             case Regular :
@@ -2764,27 +2934,27 @@ void EditView::renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches,View
               p->drawLine(x1,y1,cx,cy);
               break;
             case BWSymbols:
-              p->drawText(br3q, AlignCenter, s, 1);
+              p->drawText(m_renderBR3Cell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(br3q, AlignCenter, s, 1);
+              p->drawText(m_renderBR3Cell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->drawPolygon(QPointArray(5,br3qC));
+              p->drawPolygon(QPointArray(5,m_renderBR3Q));
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(br3q, AlignCenter, s, 1);
+                p->drawText(m_renderBR3Cell, AlignCenter, s, 1);
               }
               break;
           }
-          p->setFont(originalFont);
           break;
         case Stitch::Full:
+          p->setFont(m_renderFont);
           switch(viewMode)
           {
             case Regular :
@@ -2792,21 +2962,21 @@ void EditView::renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches,View
               p->drawLine(x1,y,x,y1);
               break;
             case BWSymbols:
-              p->drawText(rect, AlignCenter, s, 1);
+              p->drawText(m_renderCell, AlignCenter, s, 1);
               break;
             case ColorSymbols:
-              p->drawText(rect, AlignCenter, s, 1);
+              p->drawText(m_renderCell, AlignCenter, s, 1);
               break;
             case ColorBlocks:
             case ColorBlocksSymbols:
-              p->fillRect(rect,c);
+              p->fillRect(m_renderCell,c);
               if (viewMode==ColorBlocksSymbols)
               {
                 if(qGray(c.rgb()) < 128)
                   p->setPen(white);
                 else
                   p->setPen(black);
-                p->drawText(rect, AlignCenter, s, 1);
+                p->drawText(m_renderCell, AlignCenter, s, 1);
               }
               break;
             default :
