@@ -25,6 +25,7 @@ class KPrinter;
 class QRect;
 class QPainter;
 class QMimeSource;
+class QKeyEvent;
 
 /** Editor window for modifying patterns
   */
@@ -99,8 +100,20 @@ signals:
     @param enable boolean value indicating if the mask is enabled or not
   */
   void colorMask(bool enable);
+/** emitted when the status of the exclude backstitches mask has changed
+    @param enable boolean value indicating if the mask is enabled or not
+  */
+  void excludeBackstitches(bool enable);
+/** emitted when the status of the exclude knots mask has changed
+    @param enable boolean value indicating if the mask is enabled or not
+  */
+  void excludeKnots(bool enable);
 
 protected:
+/** accept some keyboard events
+    @param event QKeyEvent defining the key pressed
+  */
+  virtual void keyPressEvent(QKeyEvent* event);
 /** draw the visible parts of the scrollview that are not part of the contents,
     e.g. the scales.
     @param event QPaintEvent defining the area to be repainted.
@@ -198,6 +211,12 @@ private slots:
 /** zoom out on the editor window
   */
   void slotZoomOut();
+/** undo the last operation
+  */
+  void slotEditUndo();
+/** redo the last undone operation
+  */
+  void slotEditRedo();
 /** put the marked text/object into the clipboard and remove
     it from the document
   */
@@ -223,6 +242,12 @@ private slots:
 /** insert a library pattern
   */
   void slotLibraryPattern();
+/** export a library
+  */
+  void slotLibraryExport();
+/** import a library
+  */
+  void slotLibraryImport();
 /** insert a text string
   */
   void slotTextTool();
@@ -284,6 +309,12 @@ private slots:
 /** toggle the colorMask flag
   */
   void slotColorMask();
+/** toggle the excludeBackstitches flag
+  */
+  void slotExcludeBackstitches();
+/** toggle the excludeKnots flag
+  */
+  void slotExcludeKnots();
 /** display the full width of the document
   */
   void slotFitWidth();
@@ -348,6 +379,9 @@ private:
     @param e End point
   */
   void drawBoundingLine(QPoint s, QPoint e);
+/** initialise the rendering cells and fonts
+  */
+  void initialiseRenderCellsFonts(QPainter* p, int size);
 /** draw a bounding rectangle
     @param s TopLeft corner
     @param e BottomRight corner
@@ -369,10 +403,12 @@ private:
     @param pStitches Pointer to a Queue of Stitch
     @param viewMode viewMode to be used to render the Queue
   */
-  void renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches, ViewMode viewMode);
+  void renderCell(QPainter *p, Stitch::Queue *pStitches, ViewMode viewMode);
 
   KAction*        m_zoomIn;
   KAction*        m_zoomOut;
+  KAction*        m_editUndo;
+  KAction*        m_editRedo;
   KAction*        m_editCut;
   KAction*        m_editCopy;
   KAction*        m_editPaste;
@@ -416,6 +452,8 @@ private:
   Stitch::Type    m_currentStitchType;
   bool            m_stitchMask;
   bool            m_colorMask;
+  bool            m_excludeBackstitches;
+  bool            m_excludeKnots;
   ToolMode        m_oldMode;
   ToolMode        m_mode;
   bool            m_updatingDisplay;
@@ -436,6 +474,35 @@ private:
   QPoint          m_start;
   QPoint          m_tracking;
   QPoint          m_end;
+
+  /** QRect and QFonts used for rendering
+    */
+  QRect           m_renderCell;
+  QRect           m_renderTLCell;
+  QRect           m_renderTL3Cell;
+  QRect           m_renderTRCell;
+  QRect           m_renderTR3Cell;
+  QRect           m_renderBLCell;
+  QRect           m_renderBL3Cell;
+  QRect           m_renderBRCell;
+  QRect           m_renderBR3Cell;
+
+  QCOORD          m_renderTLQ[6];
+  QCOORD          m_renderTRQ[6];
+  QCOORD          m_renderBLQ[6];
+  QCOORD          m_renderBRQ[6];
+
+  QCOORD          m_renderBLTRH[12];
+  QCOORD          m_renderTLBRH[12];
+
+  QCOORD          m_renderTL3Q[10];
+  QCOORD          m_renderTR3Q[10];
+  QCOORD          m_renderBL3Q[10];
+  QCOORD          m_renderBR3Q[10];
+
+  QFont           m_renderFont;
+  QFont           m_renderQtrFont;
+  QFont           m_render3QtrFont;
 };
 
 #endif
