@@ -38,21 +38,11 @@ public:
     @param name Name of the scheme to be used
   */
   bool changeScheme(QString name);
-/** Displays a dialog with the currently unused colors from the current floss list and allows user selections
-    which are then added to the palette
-    @return TRUE if colors are added
-  */
-  bool addColors();
 /** Adds a specific color to the palette, first checks the palette to see if the color already exists
     @return color index of added color
     @param c @ref QColor to be added
   */
   int addColor(QColor color);
-/** Remove colors from the palette
-    @return TRUE is colors have been removed, FALSE otherwise
-    @param usedColors A list of the indexes used in the pattern.
-  */
-  bool removeColors(QValueList<int> usedColors);
 /** Remove unused colors from the palette
     @return TRUE if colors removed, FALSE otherwise
     @param usedColors A list of the indexes used in the pattern
@@ -103,8 +93,19 @@ public:
   Floss *flossAt(int i);
 /** Get the symbol associated with a floss index
     @return QChar symbol
+    @param i index of the Floss entry required
   */
   QChar symbolAt(int i);
+/** Get the number of strands to be used for this index
+    @return Number of strands
+    @param i index of the Floss entry required
+  */
+  int stitchStrandsAt(int i);
+/** Get the number of backstitch strands to be used for this index
+    @return Number of strands
+    @param i index of the Floss entry required
+  */
+  int backstitchStrandsAt(int i);
 /** Get the colormap
     @return a QMap of the floss palette
   */
@@ -132,11 +133,15 @@ public:
   */
   bool savePalette(QString schemeName);
 
+  friend class PaletteManagerDialog;
+
 private:
   int               m_current;
   QString           m_schemeName;
   QMap<int,Floss*>  m_palette;
   QMap<int,QChar>   m_symbols;
+  QMap<int,int>     m_stitchStrands;
+  QMap<int,int>     m_backstitchStrands;
 };
 
 #endif
