@@ -46,6 +46,8 @@
 #define ID_SCALE_FORMAT 3
 #define ID_STITCH_MASK 4
 #define ID_COLOR_MASK 5
+#define ID_BACKSTITCH_MASK 6
+#define ID_KNOTS_MASK 7
 
 KXStitchApplication::KXStitchApplication()
   : KApplication(),
@@ -69,6 +71,8 @@ KXStitchApp::KXStitchApp(QWidget* , const char* name)
   connect(statusBar(),SIGNAL(pressed(int)),this,SLOT(slotStatusBarClicked(int)));
   connect(m_view->m_editor,SIGNAL(stitchMask(bool)),this,SLOT(slotStitchMask(bool)));
   connect(m_view->m_editor,SIGNAL(colorMask(bool)),this,SLOT(slotColorMask(bool)));
+  connect(m_view->m_editor,SIGNAL(excludeBackstitches(bool)),this,SLOT(slotExcludeBackstitches(bool)));
+  connect(m_view->m_editor,SIGNAL(excludeKnots(bool)),this,SLOT(slotExcludeKnots(bool)));
 
   m_view->configure();
 
@@ -131,6 +135,10 @@ void KXStitchApp::initStatusBar()
   statusBar()->changeItem("",ID_STITCH_MASK);
   statusBar()->insertFixedItem("C",ID_COLOR_MASK,true);
   statusBar()->changeItem("",ID_COLOR_MASK);
+  statusBar()->insertFixedItem("B",ID_BACKSTITCH_MASK,true);
+  statusBar()->changeItem("",ID_BACKSTITCH_MASK);
+  statusBar()->insertFixedItem("K",ID_KNOTS_MASK,true);
+  statusBar()->changeItem("",ID_KNOTS_MASK);
 }
 
 void KXStitchApp::initDocument()
@@ -156,6 +164,7 @@ void KXStitchApp::openDocumentFile(const KURL& url)
     m_fileOpenRecent->addURL( url );
   slotClearBackgroundImage();
   m_view->resetView();
+  setCaption(m_doc->URL().fileName(),false);
   slotStatusMsg(i18n("Ready."));
 }
 
@@ -566,12 +575,26 @@ void KXStitchApp::slotColorMask(bool enable)
   statusBar()->changeItem(enable?"C":"",ID_COLOR_MASK);
 }
 
+void KXStitchApp::slotExcludeBackstitches(bool enable)
+{
+  statusBar()->changeItem(enable?"B":"",ID_BACKSTITCH_MASK);
+}
+
+void KXStitchApp::slotExcludeKnots(bool enable)
+{
+  statusBar()->changeItem(enable?"K":"",ID_KNOTS_MASK);
+}
+
 void KXStitchApp::slotStatusBarClicked(int item)
 {
   if (item == ID_COLOR_MASK)
     actionCollection()->action("ColorMask")->activate();
   if (item == ID_STITCH_MASK)
     actionCollection()->action("StitchMask")->activate();
+  if (item == ID_BACKSTITCH_MASK)
+    actionCollection()->action("ExcludeBackstitches")->activate();
+  if (item == ID_KNOTS_MASK)
+    actionCollection()->action("ExcludeKnots")->activate();
 }
 
 void KXStitchApp::slotSettingsConfigure()
