@@ -37,15 +37,15 @@
 #include "kxstitchmimesource.h"
 #include "patternpropertiesdialog.h"
 #include "flosslistboxitem.h"
-#include "addcolordlg.h"
 #include "floss.h"
 #include "flossscheme.h"
 #include "patterncanvas.h"
 #include "flosspalette.h"
 #include "importimagedlg.h"
 #include "configuration.h"
+#include "palettemanagerdialog.h"
 
-#define FILE_FORMAT_VERSION 5
+#define FILE_FORMAT_VERSION 6
 
 QPtrList<KXStitchView> *KXStitchDoc::viewList = 0L;
 
@@ -401,6 +401,7 @@ bool KXStitchDoc::readKXStitchDocument(QDataStream& s)
     s >> version;
     switch (version)
     {
+      case 6: // no change for this section
       case 5:
         s >> *m_propertiesDialog;
         validRead = m_palette->readKXStitchPalette(s,version) && m_canvas->readKXStitchCanvas(s,version);
@@ -541,9 +542,11 @@ bool KXStitchDoc::openDocument(const KURL& url, const char*)
     f.close();
     KIO::NetAccess::removeTempFile( tmpfile );
   }
-  if (!validRead)
+  if (validRead)
+    setURL(url);
+  else
   {
-    KMessageBox::sorry(0,i18n("KXStich could not open the file specified."),i18n("File open error"));
+    KMessageBox::sorry(0,i18n("KXStitch could not open the file specified."),i18n("File open error"));
     newDocument();
   }
   renderPixmap();
@@ -555,13 +558,16 @@ bool KXStitchDoc::importImage()
 {
   newDocument();
   KURL url;
+  QString tmpfile;
 
   url = KFileDialog::getImageOpenURL(QString::null,(KXStitchApp *)parent(),i18n("Import Image"));
   if (!url.path().isNull())
   {
-    Magick::Image magickImage(url.path().latin1());
+    KIO::NetAccess::download(url,tmpfile);
+    Magick::Image magickImage(tmpfile);
     convertImage(magickImage);
     m_propertiesDialog->setTitle(url.fileName());
+    KIO::NetAccess::removeTempFile(tmpfile);
     return true;
   }
   return false;
@@ -695,13 +701,7 @@ bool KXStitchDoc::saveDocument(const KURL& url, const char*)
   {
     QDataStream s(&f);
     /** write data to file */
-    // write file magic number and format version
-    s << QString("KXStitch");
-    s << (Q_INT16)FILE_FORMAT_VERSION;
-    // write contents of m_propertiesDialog using portable types
-    s << *m_propertiesDialog;
-    m_palette->writePalette(s);
-    m_canvas->writeCanvas(s);
+    writeKXStitchStream(s);
     f.flush();
     f.close();
   }
@@ -709,6 +709,60 @@ bool KXStitchDoc::saveDocument(const KURL& url, const char*)
   return true;
 }
 
+void KXStitchDoc::writeKXStitchStream(QDataStream &s)
+{
+  // write file magic number and format version
+  s << QString("KXStitch");
+  s << (Q_INT16)FILE_FORMAT_VERSION;
+  // write contents of m_propertiesDialog using portable types
+  s << *m_propertiesDialog;
+  m_palette->writePalette(s);
+  m_canvas->writeCanvas(s);
+}
+
+void KXStitchDoc::createSavePoint()
+{
+  QByteArray a;
+  QDataStream s(a, IO_WriteOnly);
+  writeKXStitchStream(s);
+  QByteArray *b = new QByteArray();
+  *b = qCompress(a);
+  m_undoStack.push(b);
+}
+
+bool KXStitchDoc::undo()
+{
+  createSavePoint();
+  m_redoStack.push(m_undoStack.pop());
+  KURL url = m_docURL;
+  m_propertiesDialog->reset();
+  m_palette->clear();
+  m_canvas->clear();
+  QByteArray *a = m_undoStack.pop();
+  QDataStream s(qUncompress(*a), IO_ReadOnly);
+  readKXStitchDocument(s);
+  renderPixmap();
+  m_docURL = url;
+  slotResetAllViews();
+  return !m_undoStack.isEmpty();
+}
+
+bool KXStitchDoc::redo()
+{
+  createSavePoint();
+  KURL url = m_docURL;
+  m_propertiesDialog->reset();
+  m_palette->clear();
+  m_canvas->clear();
+  QByteArray *a = m_redoStack.pop();
+  QDataStream s(qUncompress(*a), IO_ReadOnly);
+  readKXStitchDocument(s);
+  renderPixmap();
+  m_docURL = url;
+  slotResetAllViews();
+  return !m_redoStack.isEmpty();
+}
+
 void KXStitchDoc::deleteContents()
 {
   m_docURL.setFileName("");
@@ -776,10 +830,12 @@ int KXStitchDoc::findColor(QPoint cell, Stitch::Type type=Stitch::Delete)
   return m_canvas->findColor(cell, type);
 }
 
+/*
 bool KXStitchDoc::removeColors()
 {
   return m_palette->removeColors(m_canvas->usedColors());
 }
+*/
 
 bool KXStitchDoc::removeUnusedColors()
 {
@@ -798,6 +854,22 @@ void KXStitchDoc::replaceColor(int a)
   renderPixmap();
 }
 
+bool KXStitchDoc::paletteManager()
+{
+  bool added = false;
+  PaletteManagerDialog *pDlg = new PaletteManagerDialog(0, m_palette, m_canvas->usedColors());
+  if (pDlg)
+  {
+    if (pDlg->exec() == QDialog::Accepted)
+    {
+      // do stuff
+      added = true;
+    }
+    delete pDlg;
+  }
+  return added;
+}
+
 void KXStitchDoc::addStitch(int x, int y, Stitch::Type type, int i)
 {
   if (i != -1)
@@ -842,7 +914,7 @@ void KXStitchDoc::deleteBackstitch(BackStitch *bs)
   setModified();
 }
 
-QByteArray KXStitchDoc::copySelection(QRect area, Stitch::Type stitchMask, bool colorMask, bool cut)
+QByteArray KXStitchDoc::copySelection(QRect area, Stitch::Type stitchMask, bool colorMask, bool excludeBackstitches, bool excludeKnots, bool cut)
 {
   QByteArray a;
   QDataStream stream(a,IO_WriteOnly);
@@ -896,31 +968,41 @@ QByteArray KXStitchDoc::copySelection(QRect area, Stitch::Type stitchMask, bool
   int hs = area.height()*2+2;
   QPoint tl(xs,ys);
   QRect snapRect(xs,ys,ws,hs);
-  QPtrListIterator<BackStitch>* bs = m_canvas->backstitches();
-  QPtrList<BackStitch> addBS;
-  for (BackStitch* b = 0 ; (b = bs->current()) ; ++(*bs))
-    if (snapRect.contains(b->start) && snapRect.contains(b->end))
-      addBS.append(b);
-  stream << (Q_INT32)addBS.count();
-  QPtrListIterator<BackStitch> addBSit(addBS);
-  for (BackStitch* b = 0 ; (b = addBSit.current()) ; ++addBSit)
+  if (excludeBackstitches)
+    stream << (Q_INT32)0; // no backstitches to copy
+  else
   {
-    stream << (b->start-tl) << (b->end-tl) << (m_palette->flossAt(b->floss))->color;
-    if (cut)
-      m_canvas->deleteBackstitch(b);
+    QPtrListIterator<BackStitch>* bs = m_canvas->backstitches();
+    QPtrList<BackStitch> addBS;
+    for (BackStitch* b = 0 ; (b = bs->current()) ; ++(*bs))
+      if (snapRect.contains(b->start) && snapRect.contains(b->end))
+        addBS.append(b);
+    stream << (Q_INT32)addBS.count();
+    QPtrListIterator<BackStitch> addBSit(addBS);
+    for (BackStitch* b = 0 ; (b = addBSit.current()) ; ++addBSit)
+    {
+      stream << (b->start-tl) << (b->end-tl) << (m_palette->flossAt(b->floss))->color;
+      if (cut)
+        m_canvas->deleteBackstitch(b);
+    }
   }
-  QPtrListIterator<Knot>* knots = m_canvas->knots();
-  QPtrList<Knot> addKnot;
-  for (Knot* k = 0 ; (k = knots->current()) ; ++(*knots))
-    if (snapRect.contains(k->pos))
-      addKnot.append(k);
-  stream << (Q_INT32)addKnot.count();
-  QPtrListIterator<Knot> addKnotit(addKnot);
-  for (Knot* k = 0 ; (k = addKnotit.current()) ; ++addKnotit)
+  if (excludeKnots)
+    stream << (Q_INT32)0; // no knots to copy
+  else
   {
-    stream << (k->pos-tl) << (m_palette->flossAt(k->floss))->color;
-    if (cut)
-      m_canvas->deleteFrenchKnot(k->pos);
+    QPtrListIterator<Knot>* knots = m_canvas->knots();
+    QPtrList<Knot> addKnot;
+    for (Knot* k = 0 ; (k = knots->current()) ; ++(*knots))
+      if (snapRect.contains(k->pos))
+        addKnot.append(k);
+    stream << (Q_INT32)addKnot.count();
+    QPtrListIterator<Knot> addKnotit(addKnot);
+    for (Knot* k = 0 ; (k = addKnotit.current()) ; ++addKnotit)
+    {
+      stream << (k->pos-tl) << (m_palette->flossAt(k->floss))->color;
+      if (cut)
+        m_canvas->deleteFrenchKnot(k->pos);
+    }
   }
   renderPixmap();
   return a;
