@@ -19,6 +19,7 @@
 #include <qstring.h>
 #include <qptrlist.h>
 #include <qptrvector.h>
+#include <qptrstack.h>
 #include <qpointarray.h>
 #include <kurl.h>
 #include "kxstitch.h"
@@ -116,6 +117,22 @@ public:
     @param url The URL of the document
   */
   bool saveDocument(const KURL& url, const char *format=0);
+/** writes the current document to a stream for saving and creating save points
+    @param s &ref QDataStream for the data to be written to
+  */
+  void writeKXStitchStream(QDataStream &);
+/** create a save point, a copy of the current pattern in a compressed format
+    the copy will be pushed onto the undo or redo stack
+  */
+  void createSavePoint();
+/** undo the last operation allowing the option of redo
+    @return true if the undo stack still has elements on it
+  */
+  bool undo();
+/** redo the last operation that was undone, allowing the option for undo again
+    @return true if the redo stack still has elements on it
+  */
+  bool redo();
 /** modify the properties of the document
   */
   void fileProperties();
@@ -181,9 +198,11 @@ public:
     @param area A QRect giving the location and size of the selection area
     @param stitchMask The type of stitch to include, defaults to all stitch types
     @param colorMask Limit the stitches selected to the current colour if TRUE
+    @param excludeBackstitches Don't copy the backstitches if TRUE
+    @param excludeKnots Don't copy the knots if TRUE
     @param cut Cut the stitches from the pattern if TRUE
   */
-  QByteArray copySelection(QRect area, Stitch::Type stitchMask, bool colorMask, bool cut);
+  QByteArray copySelection(QRect area, Stitch::Type stitchMask, bool colorMask, bool excludeBackstitches, bool excludeKnots, bool cut);
 /** paste the selected area
     @param p QPoint defining start cell to be written over
     @param data QByteArray containing the encoded data
@@ -201,7 +220,7 @@ public:
   int findColor(QPoint cell,Stitch::Type);
 /** remove colors from palette
   */
-  bool removeColors();
+//  bool removeColors();
 /** remove all unused colors from palette
   */
   bool removeUnusedColors();
@@ -211,6 +230,9 @@ public:
 /** replace color
   */
   void replaceColor(int);
+/** initiate the palette manager
+  */
+  bool paletteManager();
 /** return a pointer to the pixmap
   */
   QPixmap *pixmap();
@@ -261,6 +283,8 @@ private:
   QPixmap*                  m_preview;
   FlossPalette*             m_palette;
   PatternCanvas*            m_canvas;
+  QPtrStack<QByteArray>     m_undoStack;
+  QPtrStack<QByteArray>     m_redoStack;
 };
 
 #endif // XSTITCHDOC_H
