@@ -13,6 +13,8 @@
 #include <klocale.h>
 #include "kxstitch.h"
 
+#include <kdebug.h>
+
 static const char *description =
   I18N_NOOP("KXStitch");
 
@@ -48,7 +50,7 @@ int main(int argc, char *argv[])
 
     if (args->count())
     {
-      kxstitch->openDocumentFile(args->arg(0));
+      kxstitch->openDocumentFile(args->url(0).url());
       args->clear();
     }
   }
