@@ -106,6 +106,9 @@ private slots:
 /** clear the unused colors from the palette
   */
   void slotClearUnused();
+/** palette manager
+  */
+  void slotPaletteManager();
 
 private:
   KAction*          m_savePalette;
